@@ -25,6 +25,7 @@
 #include <errno.h>
 #include <config.h>
 #include <math.h>
+#include <time.h>
 #include <gnome.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
@@ -40,8 +41,8 @@
 
 /* Default size for windows */
 
-#define DEFAULT_WINDOW_WIDTH 200
-#define DEFAULT_WINDOW_HEIGHT 200
+#define DEFAULT_WINDOW_WIDTH 310
+#define DEFAULT_WINDOW_HEIGHT 280
 
 #define EOG_VIEWER_CONTROL_IID "OAFIID:GNOME_EOG_Control"
 #define EOG_WINDOW_DND_POPUP_PATH  "/popups/dragndrop"
@@ -51,12 +52,21 @@ struct _EogWindowPrivate {
 	/* Our GConf client */
 	GConfClient *client;
 
+	/* URI we are displaying */
+	char *uri;
+
 	/* control frame */
-	BonoboControlFrame *ctrl_frame;
+	BonoboControlFrame  *ctrl_frame;
+
+	/* UI component - arn't these comments banal */
+	BonoboUIComponent   *ui_comp;
 
 	/* vbox */
 	GtkWidget           *box;
 
+	/* preference dialog, if available */
+	GtkWidget           *pref_dlg;
+
 	/* statusbar */
 	GtkWidget *statusbar;
 
@@ -92,6 +102,38 @@ enum {
 
 
 
+/* Computes a unique ID string for use as the window role */
+static char *
+gen_role (void)
+{
+        char *ret;
+	static char *hostname;
+	time_t t;
+	static int serial;
+
+	t = time (NULL);
+
+	if (!hostname) {
+		static char buffer [512];
+
+		if ((gethostname (buffer, sizeof (buffer) - 1) == 0) &&
+		    (buffer [0] != 0))
+			hostname = buffer;
+		else
+			hostname = "localhost";
+	}
+
+	ret = g_strdup_printf ("eog-window-%d-%d-%d-%ld-%d@%s",
+			       getpid (),
+			       getgid (),
+			       getppid (),
+			       (long) t,
+			       serial++,
+			       hostname);
+
+	return ret;
+}
+
 /* Brings attention to a window by raising it and giving it focus */
 static void
 raise_and_focus (GtkWidget *widget)
@@ -101,22 +143,36 @@ raise_and_focus (GtkWidget *widget)
 	gtk_widget_grab_focus (widget);
 }
 
+static void
+preference_dlg_closed_cb (GtkWidget *dlg, gpointer data)
+{
+	EogWindow *window;
+
+	window = EOG_WINDOW (data);
+	window->priv->pref_dlg = NULL;
+}
+
 /* Settings/Preferences callback */
 static void
 verb_Preferences_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
 	EogWindow *window;
-	EogPreferences *preferences;
+	EogWindowPrivate *priv;
 
 	g_return_if_fail (user_data != NULL);
 	g_return_if_fail (EOG_IS_WINDOW (user_data));
 
 	window = EOG_WINDOW (user_data);
+	priv = window->priv;
 
-	preferences = eog_preferences_new (window);
+	if (priv->pref_dlg == NULL) {
+		priv->pref_dlg = eog_preferences_new (window);
+		g_signal_connect (priv->pref_dlg, "destroy", 
+				  G_CALLBACK (preference_dlg_closed_cb),
+				  window);
+	}
 
-	if (preferences != NULL)
-		gnome_dialog_run_and_close (GNOME_DIALOG (preferences));
+	gtk_window_present (GTK_WINDOW (priv->pref_dlg));
 }
 
 static void
@@ -171,8 +227,15 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname
 		"Lutz Mueller",
 		NULL
 	};
+	const char *translator_credits;
 
 	if (!about) {
+		/* Translators should localize the following string
+		 * which will be displayed at the bottom of the about
+		 * box to give credit to the translator(s).
+		 */
+		translator_credits = _("Translator Credits");
+
 		about = gnome_about_new (
 			_("Eye of Gnome"),
 			VERSION,
@@ -180,11 +243,11 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname
 			_("The GNOME image viewing and cataloging program"),
 			authors,
 			NULL, /* char **documentors */
-			NULL, /* char *translators */
+			translator_credits,
 			NULL);
-		gtk_signal_connect (GTK_OBJECT (about), "destroy",
-				    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-				    &about);
+		g_signal_connect (about, "destroy",
+				  G_CALLBACK (gtk_widget_destroyed),
+				  &about);
 	}
 
 	gtk_widget_show_now (about);
@@ -221,7 +284,7 @@ activate_uri_cb (BonoboControlFrame *control_frame, const char *uri, gboolean re
 
 	window = EOG_WINDOW (eog_window_new ());
 	
-	if (g_strncasecmp ("file:", uri, 5) == 0) 		
+	if (g_ascii_strncasecmp ("file:", uri, 5) == 0) 		
 		path = g_strdup ((uri+5));
 	else
 		path = g_strdup (uri);
@@ -275,6 +338,11 @@ eog_window_destroy (GtkObject *object)
 	window = EOG_WINDOW (object);
 	priv = window->priv;
 
+	if (priv->uri) {
+		g_free (priv->uri);
+		priv->uri = NULL;
+	}
+
 	if (priv->ctrl_frame != NULL) {
 		bonobo_object_unref (BONOBO_OBJECT (priv->ctrl_frame));
 		priv->ctrl_frame = NULL;
@@ -351,10 +419,18 @@ static void
 eog_window_init (EogWindow *window)
 {
 	EogWindowPrivate *priv;
+	char *role;
 
 	priv = g_new0 (EogWindowPrivate, 1);
 	window->priv = priv;
 
+	role = gen_role ();
+	gtk_window_set_role (GTK_WINDOW (window), role);
+
+	priv->uri = NULL;
+
+	priv->pref_dlg = NULL;
+	
 	priv->client = gconf_client_get_default ();
 
 	gconf_client_add_dir (priv->client, "/apps/eog",
@@ -371,7 +447,7 @@ eog_window_init (EogWindow *window)
 
 	window_list = g_list_prepend (window_list, window);
 
-	gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);
+	g_object_set (G_OBJECT (window), "allow_shrink", TRUE, NULL);
 }
 
 /* delete_event handler for windows */
@@ -432,8 +508,6 @@ eog_window_drag_data_received (GtkWidget *widget,
 {
 	EogWindow *window;
 	EogWindowPrivate *priv;
-	GList *filenames = NULL;
-	GList *l;
 	gboolean need_new_window = TRUE;
 
 	window = EOG_WINDOW (widget);
@@ -568,8 +642,6 @@ eog_window_construct (EogWindow *window)
 {
 	EogWindowPrivate *priv;
 	BonoboUIContainer *ui_container;
-	BonoboUIComponent *ui_comp;
-	GdkGeometry geometry;
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EOG_IS_WINDOW (window));
@@ -587,12 +659,12 @@ eog_window_construct (EogWindow *window)
 	bonobo_window_set_contents (BONOBO_WINDOW (window), priv->box);
 
 	/* add menu and toolbar */
-	ui_comp = bonobo_ui_component_new ("eog");
-	bonobo_ui_component_set_container (ui_comp, 
+	priv->ui_comp = bonobo_ui_component_new ("eog");
+	bonobo_ui_component_set_container (priv->ui_comp, 
 					   BONOBO_OBJREF (ui_container), NULL);
 
-	bonobo_ui_util_set_ui (ui_comp, NULL, "eog-shell-ui.xml", "EOG", NULL);
-	bonobo_ui_component_add_verb_list_with_data (ui_comp, eog_app_verbs, window);
+	bonobo_ui_util_set_ui (priv->ui_comp, NULL, "eog-shell-ui.xml", "EOG", NULL);
+	bonobo_ui_component_add_verb_list_with_data (priv->ui_comp, eog_app_verbs, window);
 
 	/* add statusbar */
 	priv->statusbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_NEVER);
@@ -703,11 +775,10 @@ property_changed_cb (BonoboListener    *listener,
 
 	window = EOG_WINDOW (user_data);
 
-	if (!g_strcasecmp (event_name, "Bonobo/Property:change:window/title"))
+	if (!g_ascii_strcasecmp (event_name, "window/title"))
 		gtk_window_set_title (GTK_WINDOW (window),
 				      BONOBO_ARG_GET_STRING (any));
-	else if (!g_strcasecmp (event_name,
-				"Bonobo/Property:change:window/status"))
+	else if (!g_ascii_strcasecmp (event_name, "window/status"))
 		gnome_appbar_set_status (GNOME_APPBAR (window->priv->statusbar),
 					 BONOBO_ARG_GET_STRING (any));		
 }
@@ -734,7 +805,7 @@ check_for_control_properties (EogWindow *window)
 	if (title != NULL) {
 		gtk_window_set_title (GTK_WINDOW (window), title);
 		g_free (title);
-		mask = g_strdup ("Bonobo/Property:change:window/title");
+		mask = g_strdup ("window/title");
 	} else {
 		g_warning (_("Control doesn't support window_title property."));
 		gtk_window_set_title (GTK_WINDOW (window), "Eye of Gnome");
@@ -748,7 +819,7 @@ check_for_control_properties (EogWindow *window)
 					 title);
 		g_free (title);
 		/* append status_text property to list of properties to listen */
-		temp = g_strjoin (",", mask, "Bonobo/Property:change:window/status", NULL);
+		temp = g_strjoin (",", mask, "window/status", NULL);
 		g_free (mask);
 		mask = temp;
 	} else {
@@ -757,10 +828,12 @@ check_for_control_properties (EogWindow *window)
 
 	/* register for further changes */
 	if (mask) {
-		g_print ("add listener: %s\n", mask);
 		bonobo_event_source_client_add_listener (pb, 
 							 (BonoboListenerCallbackFn)property_changed_cb,
-							 mask, NULL, window);
+							 mask, &ev, window);
+		if (BONOBO_EX (&ev))
+			g_warning ("add_listener failed '%s'",
+				   bonobo_exception_get_text (&ev));
 		g_free (mask);
 	} 
 
@@ -822,7 +895,7 @@ get_viewer_control (GnomeVFSURI *uri, GnomeVFSFileInfo *info)
 	
  ctrl_error:
 	if (BONOBO_EX (&ev))
-		g_error ("%s", bonobo_exception_get_text (&ev));
+		g_warning ("%s", bonobo_exception_get_text (&ev));
 
 	CORBA_exception_free (&ev);
 
@@ -950,6 +1023,7 @@ add_control_to_ui (EogWindow *window, Bonobo_Control control)
 	GtkWidget *widget;
 	EogWindowPrivate *priv;
 	CORBA_Environment ev;
+	Bonobo_PropertyControl prop_control;
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EOG_IS_WINDOW (window));
@@ -969,6 +1043,23 @@ add_control_to_ui (EogWindow *window, Bonobo_Control control)
 
 	bonobo_control_frame_control_activate (priv->ctrl_frame);
 
+	/* update sensitivity of the properties menu item */
+	prop_control = Bonobo_Unknown_queryInterface (control, 
+						      "IDL:Bonobo/PropertyControl:1.0", &ev);
+	bonobo_ui_component_set_prop (priv->ui_comp,
+				      "/commands/Preferences",
+				      "sensitive",
+				      prop_control == CORBA_OBJECT_NIL ? "0" : "1",
+				      &ev);
+	
+	bonobo_object_release_unref (prop_control, &ev);
+
+	/* enable view menu */
+	/* FIXME: We should check if the component adds anything to 
+	 *        the menu, so that we don't view an empty menu.
+	 */
+	bonobo_ui_component_set_prop (priv->ui_comp, "/menu/View", "hidden", "0", &ev);
+
 	CORBA_exception_free (&ev);
 
 	/* retrieve control properties and install listeners */
@@ -993,6 +1084,8 @@ eog_window_open (EogWindow *window, const char *text_uri)
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *info;
 	GnomeVFSURI *uri;
+	GtkWidget *dlg;
+	gchar *uri_str;
 
 	g_return_val_if_fail (window != NULL, FALSE);
 	g_return_val_if_fail (EOG_IS_WINDOW (window), FALSE);
@@ -1004,7 +1097,6 @@ eog_window_open (EogWindow *window, const char *text_uri)
 
 	/* obtain file infos */
 	info = gnome_vfs_file_info_new ();
-	g_print ("URI: %s\n", gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE));
 	result = gnome_vfs_get_file_info_uri (uri, info,
 					      GNOME_VFS_FILE_INFO_DEFAULT |
 					      GNOME_VFS_FILE_INFO_FOLLOW_LINKS |
@@ -1014,27 +1106,52 @@ eog_window_open (EogWindow *window, const char *text_uri)
 		return FALSE;
 	}
 	
+	control = CORBA_OBJECT_NIL;
+
 	/* get appropriate control */
-	if (info->type == GNOME_VFS_FILE_TYPE_REGULAR)
+	if (info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
 		control = get_viewer_control (uri, info);
-	else if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+	}
+	else if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 		control = get_collection_control (uri, info);
-	else {
-		gchar *str;
-		str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
-		g_warning ("Couldn't handle: %s.\n", str);
-		g_free (str);
-		return FALSE;
 	}
 
-	/* add it to the user interface */
-	add_control_to_ui (window, control);
-	bonobo_object_release_unref (control, NULL);
+	if (control != CORBA_OBJECT_NIL) {
+		/* add it to the user interface */
+		add_control_to_ui (window, control);
+		bonobo_object_release_unref (control, NULL);
+	}
+	else {
+		uri_str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+
+		/* FIXME: The error message should be more specific 
+		 *        (eg. "Unknown file format").
+		 */
+		dlg = gtk_message_dialog_new (GTK_WINDOW (window),
+					      GTK_DIALOG_DESTROY_WITH_PARENT,
+					      GTK_MESSAGE_ERROR,
+					      GTK_BUTTONS_CLOSE,
+					      _("Unable to open %s."),
+					      uri_str);
+
+		g_signal_connect (dlg, "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+
+		gtk_widget_show (dlg);
+
+		g_free (uri_str);
+	}
 
 	/* clean up */
 	gnome_vfs_file_info_unref (info);
 	gnome_vfs_uri_unref (uri);
 
+	if (priv->uri)
+		g_free (priv->uri);
+
+	priv->uri = g_strdup (text_uri);
+
 	return TRUE;
 }
 
@@ -1076,3 +1193,37 @@ eog_window_get_property_control (EogWindow *window, CORBA_Environment *ev)
 						      "IDL:Bonobo/PropertyControl:1.0", ev);
 	return prop_control;
 }
+
+/**
+ * eog_get_window_list:
+ *
+ * Gets a list of all #EogWindow objects.  You should not modify this list at
+ * all; it should be considered read-only.
+ * 
+ * Return value: A list with all the active #EogWindow objects.
+ **/
+GList *
+eog_get_window_list (void)
+{
+	return window_list;
+}
+
+/**
+ * eog_window_get_uri:
+ * @eog_window: A shell window.
+ * 
+ * Queries the URI that is being displayed by the specified window.  If the
+ * window is not displaying a single image or directory, this will return NULL.
+ * 
+ * Return value: The URI that is being displayed.
+ **/
+const char *
+eog_window_get_uri (EogWindow *eog_window)
+{
+	EogWindowPrivate *priv;
+
+	g_return_val_if_fail (EOG_IS_WINDOW (eog_window), NULL);
+
+	priv = eog_window->priv;
+	return priv->uri;
+}
